
/* 
   Project:     SOFTSUSY 1.2
   File:        softsusy.h
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: Header file for RP conserving MSSM object including all (real)
                soft SUSY breaking parameters and (real) SUSY couplings.

   $Log: softsusy.h,v $
   Revision 1.10  2001/10/31 09:12:08  allanach
   Altered so that an early no rho convergence can disappear

   Revision 1.9  2001/10/23 12:53:58  allanach
   Easier user interface utilised

   Revision 1.8  2001/10/03 13:34:16  allanach
   Changed name of complex.h to avoid conflict with STD libraries

   Revision 1.7  2001/09/28 13:59:58  allanach
   Split rhohat determination up into sensible pieces - small bugs fixed there.

   Revision 1.6  2001/07/30 14:08:47  allanach
   Added ISAWIG and SSRUN interface

   Revision 1.5  2001/07/26 14:20:34  allanach
   Added isajet interface

   Revision 1.4  2001/07/18 15:54:50  allanach
   Put MIXING switch into def.h

   Revision 1.3  2001/07/18 14:42:51  allanach
   Added proper header info

   */

#ifndef SOFTSUSY_H
#define SOFTSUSY_H

/* Contains all supersymmetric MSSM parameters INCLUDING SOFT TERMS and RGEs
   for interpolating them (In MSSM domain) */
#include<iostream.h>
#include<math.h>
#include<fstream.h>
#include "lowe.h"
#include "susy.h"
#include "def.h"
#include "linalg.h"
#include "mycomplex.h"
#include "physpars.h"
#include "softpars.h"

class MssmSoftsusy;
istream & operator >>(istream &left, MssmSoftsusy &s);

/* ------------------SOFTSUSY starts here ---------------------*/
class MssmSoftsusy: public SoftParsMssm
{
private:
  sPhysical physpars;
  sProblem problem;
  double minV;
  double m32; // Gravitino mass
  double higgsVevMs; // around 246.22 
  QedQcd dataSet; // contains low energy data on quark masses etc

public:
  MssmSoftsusy();
  MssmSoftsusy(const MssmSusy &);
  MssmSoftsusy(const MssmSoftsusy &);
  MssmSoftsusy(const SoftParsMssm &, const sPhysical &, double, int, int,
	       double, double); 
  const MssmSoftsusy & operator=(const MssmSoftsusy & s);
  
  inline sPhysical displayPhys() const;
  SoftParsMssm displaySoftPars() const;
  sProblem displayProblem() const {return problem; };
  inline QedQcd displayDataSet() const;
  inline double displayHiggsVevMs() const;
  double displayGravitino() const;
  int displayMinimum() const;
  double displayMinpot() const;
  double displayTadpole1Ms() const;
  double displayTadpole2Ms() const;
  
  void flagNonperturbative(bool a) { problem.nonperturbative = a; };
  void flagTachyon(bool a) { problem.tachyon = a; };
  void flagB(bool a) { problem.b = a; };
  void flagNoConvergence(bool a) { problem.noConvergence = a; };
  void flagNoMuConvergence(bool a) { problem.noMuConvergence = a; };
  void flagNoRhoConvergence(bool a) { problem.noRhoConvergence = a; };
  void flagMusqwrongsign(bool a) { problem.muSqWrongSign = a; };
  void flagHiggsufb(bool a) { problem.higgsUfb = a; };
  void flagAllProblems(bool a) { problem.tachyon = a; problem.b = a;
  problem.noConvergence = a; problem.higgsUfb = a;
  problem.nonperturbative = a; problem.noRhoConvergence = a; }
  
  void setHiggsVevMs(double v) { higgsVevMs = v; };
  void setData(const QedQcd & r) { dataSet = r; };
  void setM32(double);
  void setMinpot(double);
  void doUfb3(double mgut);
  void setPhys(const sPhysical & s) { physpars = s; };
  virtual void setTadpole1Ms(double mt, double sinthDRbar);
  virtual void setTadpole2Ms(double mt, double sinthDRbar);
  void overwriteTadpole1Ms(double value){physpars.t1OV1Ms = value;};
  void overwriteTadpole2Ms(double value){physpars.t2OV2Ms = value;};
  
  void addStopCorrection(DoubleMatrix & mass, double mt);
  void addSbotCorrection(DoubleMatrix & mass, double mt);
  void addSquarkCorrection(DoubleMatrix & mass);
  void doStop(double mt, double pizztMS, double sinthDRbarMS, int
		      accuracy); 
  void doSbot(double mb, double pizztMS, double sinthDRbarMS, int
		      accuracy, double mt);
  void doStau(double mT, double pizztMS, double sinthDRbarMS, int
		      accuracy);
  void doSnu(double pizztMS);
  void treeStop(DoubleMatrix & mass, double mtrun, double pizztMS, 
		double sinthDRbarMS, int family);
  void treeSbot(DoubleMatrix & mass, double mbrun, double pizztMS, 
		double sinthDRbarMS, int family);
  void treeStau(DoubleMatrix & mass, double mTrun, double pizztMS, 
		double sinthDRbarMS, int family);
  void treeSnu(double & mSq, double pizztMS, int family);

  virtual void sparticleThresholdCorrections(double tb);
  double qedSusythresh(double alphaEm, double Q) const;
  double qcdSusythresh(double alphasMSbar, double Q) const;
  double calcMs() const;
  virtual void physical(int accuracy);
  double calcRunningMt() const;
  double calcRunningMtau() const;  
  double calcRunningMb() const;
  double calcHt(double vev) const;
  double calcHb(double vev) const;
  double calcHtau(double vev) const;
  double calcSinthdrbar() const;
  double getVev() const;
  virtual void charginos(int accuracy, double piwwt);
  virtual void addCharginoLoop(DoubleMatrix &);
  virtual void neutralinos(int accuracy, double piwwt, double pizzt);
  virtual void addNeutralinoLoop(DoubleMatrix &);
  virtual void gluino(int accuracy);
  virtual void higgs(int, double piwwt, double pizzt);
  virtual int rewsbMu(int, double &) const;
  virtual int rewsbB(double, double &) const;
  virtual void rewsb(int, double);
  void rewsbTreeLevel(int);
  void iterateMu(double & munew, int sgnMu, double mt, 
		  int maxTries, double pizztMS, double sinthDRbar, double tol,
		 int  & err);
  double predTanb() const;
  double predMzsq(double &) const;
  DoubleVector fineTune(void (*boundaryCondition)(MssmSoftsusy &, 
						  const DoubleVector &), 
		   const DoubleVector & bcPars, double MX);
  double it1par(int numPar, const DoubleVector & bcPars);
  double ufb3sl(double);
  double realMinMs() const;
  
  virtual double piZZT(double p, double Q) const;
  virtual double piWWT(double p, double Q) const;
  //  double piZGT(double p, double Q) const;
  virtual double piAA(double p, double Q) const;
  virtual void rhohat(double & outrho, double & outsin, double alphaMZDRbar,
		      double pizztMZ, double tol, int maxTries);
  float deltaVb(double outrho, double outsin, double alphaDRbar, 
		double pizztMZ) const;
  float dRho(double outrho, double outsin, double alphaDRbar, 
		 double pizztMZ);
  float dR(double outrho, double outsin, double alphaDRbar, double pizztMZ);

  int lsp(double &, int &, int &) const;
  
  void printShort() const;
  void printLong();
  
  double thet(double a, double b, double c);
 
  void mpzNeutralinos(ComplexMatrix & n, DoubleVector & mneut) const; 
  void mpzCharginos(ComplexMatrix & u, ComplexMatrix & v, DoubleVector & mch)
    const;

  void lowOrg(void (*boundaryCondition)(MssmSoftsusy &, const DoubleVector &),
	      double mx, const DoubleVector & pars, int sgnMu, double tanb,
	      const QedQcd & oneset);

  void itLowsoft(int maxTries, double MX, int sgnMu, double tol, double tanb,
		 void (*boundaryCondition)(MssmSoftsusy &, const DoubleVector
					   &), const DoubleVector & pars);

  void methodBoundaryCondition(const DoubleVector & pars);

  void isajetNumbers751
(double & mtopPole, double & mGPole, double & smu, double & mA, double & tanb, 
 double & mq1l, double & mdr, double & mur, double & meL, double & meR, double & mql3, 
 double & mdr3, double & mur3, double &  mtauL, double & mtauR, double & at, double & ab, 
 double & atau, double & mq2l, double & msr, double & mcr, double & mmuL, double & mmuR, 
 double & m1, double & m2) const; 
  void isajetInterface751() const;
  void ssrunInterface751(char fname []) const;
  void isawigInterface751(char fnamein [80], char fnameout [80]) const;
};

inline MssmSoftsusy::MssmSoftsusy()
  : SoftParsMssm(), physpars(), problem(), minV(6.66e66), m32(0.0),
  higgsVevMs(246.22), dataSet() { 
      setPars(110);
      setMu(0.0);
      setLoops(0);
      setThresholds(0);
}


inline MssmSoftsusy::MssmSoftsusy(const MssmSoftsusy & s)
  : SoftParsMssm(s.displaySoftPars()), physpars(s.displayPhys()),
    problem(s.problem), minV(s.minV), m32(s.m32),
    higgsVevMs(s.displayHiggsVevMs()), dataSet(s.displayDataSet()) {

    setAllGauginos(s.displayGaugino());
    setTrilinearMatrix(UA, s.displayTrilinear(UA));
    setTrilinearMatrix(DA, s.displayTrilinear(DA));
    setTrilinearMatrix(EA, s.displayTrilinear(EA));

    setSoftMassMatrix(mQl, s.displaySoftMassSquared(mQl)); 
    setSoftMassMatrix(mUr, s.displaySoftMassSquared(mUr)); 
    setSoftMassMatrix(mDr, s.displaySoftMassSquared(mDr)); 
    setSoftMassMatrix(mLl, s.displaySoftMassSquared(mLl)); 
    setSoftMassMatrix(mEr, s.displaySoftMassSquared(mEr)); 

    setB(s.displayB());
    setMh1Squared(s.displayMh1Squared());
    setMh2Squared(s.displayMh2Squared());

    setPars(110);
    setMu(s.displayMu()); 
    setLoops(s.displayLoops());
    setThresholds(s.displayThresholds());
}

inline MssmSoftsusy::MssmSoftsusy(const MssmSusy &s)
  : SoftParsMssm(s), physpars(), problem(), minV(6.66e66), m32(0.0),
    higgsVevMs(246.22), dataSet() { 
      setPars(110);
      setMu(s.displayMu()); 
      setLoops(s.displayLoops());
      setThresholds(s.displayThresholds());
}

inline MssmSoftsusy::MssmSoftsusy
(const SoftParsMssm & s, const sPhysical & sp, double mu, int l, int t, double
 mg, double hv) 
  : SoftParsMssm(s), physpars(sp), problem(), minV(6.66e66), m32(mg),
    higgsVevMs(hv), dataSet() {
      setPars(110);
      setMu(mu);
      setLoops(l);
      setThresholds(t);
}

inline QedQcd MssmSoftsusy::displayDataSet() const { return dataSet; }

inline double MssmSoftsusy::displayHiggsVevMs() const { 
  return higgsVevMs; 
}

inline double MssmSoftsusy::displayGravitino() const { return m32; }

inline SoftParsMssm MssmSoftsusy::displaySoftPars() const { 
  return SoftParsMssm(displaySusy(), displayGaugino(),
		      displayTrilinear(UA),
		      displayTrilinear(DA),
		      displayTrilinear(EA), 
		      displaySoftMassSquared(mQl), 
		      displaySoftMassSquared(mUr), 
		      displaySoftMassSquared(mDr), 
		      displaySoftMassSquared(mLl), 
		      displaySoftMassSquared(mEr),
		      displayB(), displayMh1Squared(), displayMh2Squared(),
		      displayMu(), displayLoops(), displayThresholds()); 
}

inline sPhysical MssmSoftsusy::displayPhys() const { return physpars; }

inline double MssmSoftsusy::displayMinpot() const { return minV; } 

inline double MssmSoftsusy::displayTadpole1Ms() const {
  return physpars.t1OV1Ms; 
}

inline double MssmSoftsusy::displayTadpole2Ms() const {
  return physpars.t2OV2Ms; 
}

inline void MssmSoftsusy::setM32(double a) { m32 = a; }

inline void MssmSoftsusy::setMinpot(double f) { minV = f; }
inline void MssmSoftsusy::doUfb3(double mgut) { setMinpot(ufb3sl(mgut) -
							  realMinMs()); } 
// ---------------------- Other functions associated with class -------------
void printShortInitialise();
ostream & operator <<(ostream &, const MssmSoftsusy &); 
double sumTol(const MssmSoftsusy & in, const MssmSoftsusy & out);
inline double ccbSqrt(double);
void recogLsp(int, int);
double rho2(double r);
MssmSusy guessAtSusyMt(double tanb, const QedQcd & oneset);

// ufb3 direction CCB calculations
extern double minimufb3(double);
double ufb3fn(double,double,double,int,MssmSoftsusy);
double getQhat(double, double, double, double, double, MssmSoftsusy &);

#endif



